/*
 * This tool runs on NT or UNIX, not the device and can use any and all java
 */

import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.MissingResourceException;

import java.io.*;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
import java.util.Enumeration;

/**
	 Generate a .MF manifest file or a .JAD Java Application Descriptor,
	 depending on if -m or -j is passed in, respectively.

	 The manifest file has to go into the JAR file first so that the size of
	 the jar can be put into the JAD file.   Hence the 2 step process.

	 Rather than hard code the host and port number in the ANT xml file, 
	 we read from lib/JadSettings.properties, matching on the application
	 name passed in, e.g. VNurseHost and VNursePort are looked up and 
	 override the command line options.

	 This should be extended to all of the arguments, exercise for the user.

	 The top level build.xml has makeManifest and makeJad productions which
	 illustrate the use of this class.
*/

class DeployManifest
{
	private PrintWriter pw;
	private String jarFile = null;
	private String outFile = null;
	private Properties props;

	private String Host = "localhost";
	private String Port = "8080";

	private boolean makeJadFile = false;
	private boolean makeManifest = false;
	private int xferDownSize = -1;
	private int xferUpSize = -1;
	private String userName;
	private String password;
	private int numClients = 1;

	private String lang;

	private String appName;
	private String className;
	private String pngName;

	private boolean kddi = false;
	private String maxDataSize = "12000";

	/*
		Host and Port defaults
	*/

	private static final String PROP_FILE = "JadSettings";
	private static PropertyResourceBundle configProps;

	public static void main(String args[])
	{
		try
		{
			new DeployManifest(args);
		}
		catch (Exception e)
		{
		    System.out.println("DeployManifest failed: " + e.toString());
			e.printStackTrace();
			return;
		}
	}

	private void usage(String msg) throws Exception
	{
		System.out.println(msg);
	    throw new Exception("\n\tjava DeployManifest " 
				+ "{ -j jarFileName  \\ \n"
				+ " [ -p Subscription_password ] [ -u Subscription_userName ] \\ \n"
				+ " [ -n numclients ] [ -D Down_maxTransferSize] \\ \n"
				+ " [ -U Up_maxTransferSize] [ -l lang] AppName outFileName } \\"
				+ "\n\tor\n{ -m AppName ClassName PngFileName outFileName }"
				+ "\n -j and -n overrides user/password flag");
	}

	private static final int APPNAME=1;
	private static final int CLASSNAME=2;
	private static final int PNGNAME=3;
	private static final int OUTFILE=4;

	public DeployManifest(String args[]) throws Exception
	{
		if (args.length < 5)
		{
		    usage("Too few arguments");
		}

		int nextArg = APPNAME;

	    for (int a = 0; a < args.length; a++)
		{
			if (args[a].charAt(0) == '-')
			{
				char sel = args[a].charAt(1);
				switch (sel)
				{
				 case 'l':
					if (++a >= args.length)
						usage ("Must supply language (jp, en, es, fr) with -l flag");
					lang = args[a];
					break;
				 case 'j':
				    makeJadFile = true;
					if (++a >= args.length)
						usage ("Must supply jarname with -j flag");
					jarFile = args[a];
					break;
				  case 'm':
				    makeManifest = true;
					break;

				  case 'n':
					if (++a >= args.length)
						usage ("Must supply number of clients with -n flag");
					numClients = Integer.parseInt(args[a]);
					System.out.println("numClients is " + numClients);
					break;

				  case 'U':
					if (++a >= args.length)
						usage ("Must supply length with -U flag");
					xferUpSize = Integer.parseInt(args[a]);
					System.out.println("xferUpSize is " + xferUpSize);
					break;
				  case 'D':
					if (++a >= args.length)
						usage ("Must supply length with -D flag");
					xferDownSize = Integer.parseInt(args[a]);
					System.out.println("xferDownSize is " + xferDownSize);
					break;

				  case 'p':
					if (++a >= args.length)
						usage ("Must supply password with -p flag");
					password = args[a];
					System.out.println("password is " + password);
					break;
				  case 'u':
					if (++a >= args.length)
						usage ("Must supply user name with -u flag");
					userName = args[a];
					System.out.println("userName is " + userName);
					break;

				  default:
					usage("Bad flag -" + sel);
				}
				continue;
			}
			if (makeJadFile && makeManifest)
				usage("Must specify only one of -j or -m flag");
			if (nextArg == APPNAME)
			{
			    appName = args[a];
				if (makeJadFile)
					nextArg = OUTFILE;
				else
					nextArg = CLASSNAME;
			}
			else if (nextArg == CLASSNAME)
			{
				className = args[a];
				nextArg = PNGNAME;
			}
			else if (nextArg == PNGNAME)
			{
				pngName = args[a];
				nextArg = OUTFILE;
			}
			else if (nextArg == OUTFILE)
			{
			    outFile = args[a];
				nextArg = -1;
			}
			else
			    usage("Wrong number of args " + a);
		}
		if (makeJadFile)
			System.out.println("makeJadFile to " + outFile);
		if (makeManifest)
			System.out.println("makeManifest to " + outFile);
		if (makeJadFile && makeManifest)
		    usage("Must specify only one of -j or -m flag");
		if (!makeJadFile && !makeManifest)
		    usage("Must specify the -j or -m flag");
		if (appName == null || outFile == null)
		    usage("Must give required application Name and output file arguments");
		if (makeManifest && (pngName == null || className == null))
		    usage("Must give application png and class names to -j");

		int size = -1;
		if (makeJadFile)
		{
			readProperties(appName);

			FileInputStream jarIn = new FileInputStream(jarFile);
			size = jarIn.available();
			jarIn.close();
			System.out.println("jar size is " + size);

			// create one without the number in the name, for iDEN Update
			generateOneFile(outFile + ".jad", 1, size);

			for (int i = 1; i <= numClients; i++)
				generateOneFile(outFile + i + ".jad", i, size);
		}
		else
			generateOneFile(outFile, -1, -1);

	}
	private void generateOneFile(String outFile, int i, int size)
		throws Exception
	{
		System.out.println("generateOnefile " + outFile);
		pw = createOutputFile(outFile);

		/*
			From the MIDP Spec:

			The manifest MUST contain the following attributes:
				 MIDlet-Name
				 MIDlet-Version
				 MIDlet-Vendor
				 MIDlet-<n> for each MIDlet
				 MicroEdition-Profile
				 MicroEdition-Configuration

			The manifest MAY contain the following:
				 MIDlet-Description
				 MIDlet-Icon
				 MIDlet-Info-URL
				 MIDlet-Data-Size


			The application descriptor MUST contain the following attributes:
				 MIDlet-Name
				 MIDlet-Version
				 MIDlet-Vendor
				 MIDlet-Jar-URL
				 MIDlet-Jar-Size

			The application descriptor MAY contain:
				 MIDlet-Description
				 MIDlet-Icon
				 MIDlet-Info-URL
				 MIDlet-Data-Size
				 MIDlet specific attributes that do not begin with "MIDlet-"

		*/

		pw.println("MIDlet-Name: " + appName);
		pw.println("MIDlet-Version: 1.0");
		pw.println("MIDlet-Vendor: IBM Corp");
		pw.println("MIDlet-Description: Visiting Nurse Demo");

		if (makeManifest)
		{
			pw.println("MIDlet-1: " + appName + ", " + pngName + ", " +
				className);
			pw.println("MicroEdition-Profile: MIDP-1.0");
			pw.println("MicroEdition-Configuration: CLDC-1.0");
		}
		else if (makeJadFile)
		{
			pw.println("MIDlet-Jar-URL: " + appName + ".jar");
			pw.println("MIDlet-Jar-Size: " + size);
			if (xferDownSize > 0)
				pw.println("PacketDownSize: " + xferDownSize);
			if (xferUpSize > 0)
				pw.println("PacketUpSize: " + xferUpSize);
			if (userName != null)
				pw.println("Db2eSyncUserName: " + userName);
			else
				pw.println("Db2eSyncUserName: nurse" + i);
			if (password != null)
				pw.println("Db2eSyncPassword: " + password);
			else
				pw.println("Db2eSyncPassword: nurse" + i);
			String URI = "http://" + Host;
			if (Port != null)
				URI += ":" + Port;
			pw.println("Db2eSyncURI: " + URI);
			if (lang != null)
				pw.println("Db2eSyncLang: " + lang);

			if (kddi)
			{
				pw.println("MicroEdition-Configuration: CLDC-1.0");
				pw.println("MicroEdition-Profile: MIDP-1.0");
				pw.println("MIDlet-X-AllowURL-1: http://" + Host
					+ ":" + Port + "/");
				pw.println("MIDlet-Data-Size: " + maxDataSize);
			}
		}

		pw.close();
	}

	private PrintWriter writeOutputFile(String outputFile) 
	{
	    FileOutputStream stream = null;
	    try
		{
		    stream = new FileOutputStream(outputFile);
		}
		catch (Exception e)
		{
		    System.out.println("Could not open output file: " + outputFile 
				+ "\n\t" + e.toString());
			return null;
		}
		return new PrintWriter(stream);
	}
	
	private PrintWriter createOutputFile(String outputFile) throws Exception
	{
	    FileOutputStream stream = null;
	    try
		{
		    stream = new FileOutputStream(outputFile);
		}
		catch (Exception e)
		{
		    throw new Exception("Could not open output file: " + outputFile 
				+ "\n\t" + e.toString());
		}
		return new PrintWriter(stream);
	}

	/**
		Override the defaults via a property file
	*/

	private void readProperties(String appName)
	{
		configProps = loadProperties();
		if (configProps == null)
			return;

		System.out.println("Loaded properties");

		System.out.println("readProp: " + appName);

		String x = getProp("Phone");
		if (x != null && x.equalsIgnoreCase("KDDI"))
			kddi = true;
		x = getProp("DataSize");
		if (x != null)
			maxDataSize = x;

		x = getProp(appName + "Host");
		if (x != null)
			Host = x;
		x = getProp(appName + "Port");
		if (x != null)
			Port = x;

		System.out.println("Config: Host " + Host + " Port " + Port);
	}
	private String getProp(String key)
	{
		try
		{
			if (configProps == null)
				return null;
			return configProps.getString(key);
		}
		catch (MissingResourceException mre)
		{
			return null;
		}
	}
	/**
		Loads the properties from the file.

		@param propFile location within classpath of the property file
	 */
	private static boolean loaded = false;
	private static PropertyResourceBundle loadProperties()
	{
		if (!loaded)
		{
			configProps =
				 (PropertyResourceBundle)ResourceBundle.getBundle(PROP_FILE);
			loaded = true;
		}
		return configProps;
	}
}
